@@ -1,18 +1,18 @@
 #!/usr/bin/env python 
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2011 Modelon AB
+# Copyright (C) 2010 Modelon AB
 #
 # This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
+# it under the terms of the GNU Lesser General Public License as published by
 # the Free Software Foundation, version 3 of the License.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
+# GNU Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Lesser General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 import numpy as N
@@ -26,7 +26,221 @@ from assimulo.explicit_ode import Explicit_ODE
 from assimulo.implicit_ode import Implicit_ODE
 from assimulo.lib.radau_core import Radau_Common
 
+from assimulo.lib import radau5
+
 class Radau5ODE(Radau_Common,Explicit_ODE):
+    """
+    Radau IIA fifth-order three-stages with step-size control and 
+    continuous output. Based on the FORTRAN code RADAU5 by E.Hairer and 
+    G.Wanner, which can be found here: 
+    http://www.unige.ch/~hairer/software.html
+    
+    Details about the implementation (FORTRAN) can be found in the book,::
+    
+        Solving Ordinary Differential Equations II,
+        Stiff and Differential-Algebraic Problems
+        
+        Authors: E. Hairer and G. Wanner
+        Springer-Verlag, ISBN: 3-540-60452-9
+    
+    """
+    
+    def __init__(self, problem):
+        """
+        Initiates the solver.
+        
+            Parameters::
+            
+                problem     
+                            - The problem to be solved. Should be an instance
+                              of the 'Explicit_Problem' class.
+        """
+        Explicit_ODE.__init__(self, problem) #Calls the base class
+        
+        #Default values
+        self.options["inith"]    = 0.01
+        self.options["newt"]     = 7 #Maximum number of newton iterations
+        self.options["thet"]     = 1.e-3 #Boundary for re-calculation of jac
+        self.options["fnewt"]    = 0.0 #Stopping critera for Newtons Method
+        self.options["quot1"]    = 1.0 #Parameters for changing step-size (lower bound)
+        self.options["quot2"]    = 1.2 #Parameters for changing step-size (upper bound)
+        self.options["fac1"]     = 0.2 #Parameters for step-size selection (lower bound)
+        self.options["fac2"]     = 8.0 #Parameters for step-size selection (upper bound)
+        self.options["maxh"]     = N.inf #Maximum step-size.
+        self.options["safe"]     = 0.9 #Safety factor
+        self.options["atol"]     = 1.0e-6*N.ones(self.problem_info["dim"]) #Absolute tolerance
+        self.options["rtol"]     = 1.0e-6 #Relative tolerance
+        self.options["usejac"]   = True if self.problem_info["jac_fcn"] else False
+        self.options["maxsteps"] = 10000
+        
+        # - Statistic values
+        self.statistics["nsteps"]      = 0 #Number of steps
+        self.statistics["nfcn"]        = 0 #Number of function evaluations
+        self.statistics["njac"]        = 0 #Number of jacobian evaluations
+        self.statistics["njacfcn"]     = 0 #Number of function evaluations when evaluating the jacobian
+        self.statistics["errfail"]     = 0 #Number of step rejections
+        self.statistics["nlu"]         = 0 #Number of LU decompositions
+        self.statistics["nstepstotal"] = 0 #Number of total computed steps (may NOT be equal to nsteps+nerrfail)
+        self.statistics["nstateevents"]= 0 #Number of state events
+        self.statistics["ngevals"]     = 0 #Root evaluations
+        
+        #Solver support
+        self.supports["report_continuously"] = True
+        self.supports["interpolated_output"] = True
+        self.supports["state_events"] = True
+        
+        self._leny = len(self.y) #Dimension of the problem
+        self._type = '(explicit)'
+        self._event_info = None
+        
+    def initialize(self):
+        #Reset statistics
+        for k in self.statistics.keys():
+            self.statistics[k] = 0
+            
+    def set_problem_data(self):
+        if self.problem_info["state_events"]:
+            def event_func(t, y):
+                return self.problem.state_events(t, y, self.sw)
+            def f(t, y):
+                return self.problem.rhs(t, y, self.sw)
+            self.f = f
+            self.event_func = event_func
+            self._event_info = [0] * self.problem_info["dimRoot"]
+            self.g_old = self.event_func(self.t, self.y)
+        else:
+            self.f = self.problem.rhs
+    
+    def interpolate(self, time):
+        y = N.empty(self._leny)
+        for i in range(self._leny):
+            y[i] = radau5.contr5(i+1, time, self.cont)
+        
+        return y
+    
+    def _solout(self, nrsol, told, t, y, cont, lrc, irtrn):
+        """
+        This method is called after every successful step taken by Radau5
+        """
+        self.cont = cont #Saved to be used by the interpolation function.
+        
+        if self.problem_info["state_events"]:
+            flag, t, y = self.event_locator(told, t, y)
+            #Convert to Fortram indicator.
+            if flag == ID_PY_EVENT: irtrn = -1
+            
+        if self._opts["report_continuously"]:
+            initialize_flag = self.report_solution(t, y, self._opts)
+            if initialize_flag: irtrn = -1
+        else:
+            if self._opts["output_list"] == None:
+                self._tlist.append(t)
+                self._ylist.append(y.copy())
+            else:
+                output_list = self._opts["output_list"]
+                output_index = self._opts["output_index"]
+                try:
+                    while output_list[output_index] <= t:
+                        self._tlist.append(output_list[output_index])
+                        self._ylist.append(self.interpolate(output_list[output_index]))
+                        
+                        output_index += 1
+                except IndexError:
+                    pass
+                self._opts["output_index"] = output_index
+        
+        return irtrn
+            
+    def integrate(self, t, y, tf, opts):
+        ITOL  = 1 #Both atol and rtol are vectors
+        IJAC  = 1 if self.usejac else 0 #Switch for the jacobian, 0==NO JACOBIAN
+        MLJAC = self.problem_info["dim"] #The jacobian is full
+        MUJAC = self.problem_info["dim"] #See MLJAC
+        IMAS  = 0 #The mass matrix is the identity
+        MLMAS = self.problem_info["dim"] #The mass matrix is full
+        MUMAS = self.problem_info["dim"] #See MLMAS
+        IOUT  = 1 #solout is called after every step
+        WORK  = N.array([0.0]*(4*self.problem_info["dim"]**2+12*self.problem_info["dim"]+20)) #Work (double) vector
+        IWORK = N.array([0]*(3*self.problem_info["dim"]+20)) #Work (integer) vector
+        
+        #Setting work options
+        WORK[1] = self.safe
+        WORK[2] = self.thet
+        WORK[3] = self.fnewt
+        WORK[4] = self.quot1
+        WORK[5] = self.quot2
+        WORK[6] = self.maxh
+        WORK[7] = self.fac1
+        WORK[8] = self.fac2
+        
+        #Setting iwork options
+        IWORK[1] = self.maxsteps
+        IWORK[2] = self.newt
+        
+        #Dummy methods
+        mas_dummy = lambda t:x
+        jac_dummy = (lambda t:x) if not self.usejac else self.problem.jac
+        
+        #Check for initialization
+        if opts["initialize"]:
+            self.set_problem_data()
+            self._tlist = []
+            self._ylist = []
+            
+        
+        #Store the opts
+        self._opts = opts
+        
+        t, y, h, iwork, flag =  radau5.radau5(self.f, t, y.copy(), tf, self.inith, self.rtol*N.ones(self.problem_info["dim"]), self.atol, 
+                        ITOL, jac_dummy, IJAC, MLJAC, MUJAC, mas_dummy, IMAS, MLMAS, MUMAS, self._solout, IOUT, WORK, IWORK)
+        
+        #Checking return
+        if flag == 1:
+            flag = ID_PY_COMPLETE
+        elif flag == 2:
+            flag = ID_PY_EVENT
+        else:
+            raise Exception("Radau5 failed with flag %d"%flag)
+        
+        #Retrieving statistics
+        self.statistics["nsteps"]      += iwork[16]
+        self.statistics["nfcn"]        += iwork[13]
+        self.statistics["njac"]        += iwork[14]
+        self.statistics["nstepstotal"] += iwork[15]
+        self.statistics["errfail"]     += iwork[17]
+        self.statistics["nlu"]         += iwork[18]
+        
+        return flag, self._tlist, self._ylist
+    
+    def state_event_info(self):
+        return self._event_info
+        
+    def set_event_info(self, event_info):
+        self._event_info = event_info
+    
+    def print_statistics(self, verbose=NORMAL):
+        """
+        Prints the run-time statistics for the problem.
+        """
+        self.log_message('Final Run Statistics: %s \n' % self.problem.name,        verbose)
+        
+        self.log_message(' Number of Steps                          : '+ str(self.statistics["nsteps"]),          verbose)               
+        self.log_message(' Number of Function Evaluations           : '+ str(self.statistics["nfcn"]),         verbose)
+        self.log_message(' Number of Jacobian Evaluations           : '+ str(self.statistics["njac"]),    verbose)
+        self.log_message(' Number of Error Test Failures            : '+ str(self.statistics["errfail"]),       verbose)
+        self.log_message(' Number of LU decompositions              : '+ str(self.statistics["nlu"]),       verbose)
+        if self.problem_info["state_events"]:
+            self.log_message(' Number of Root Evaluations               : '+ str(self.statistics["ngevals"]),        verbose)
+            self.log_message(' Number of State-Events                   : '+ str(self.statistics["nstateevents"]),   verbose)
+        
+        self.log_message('\nSolver options:\n',                                      verbose)
+        self.log_message(' Solver                  : Radau5 ' + self._type,          verbose)
+        self.log_message(' Tolerances (absolute)   : ' + str(self.options["atol"]),  verbose)
+        self.log_message(' Tolerances (relative)   : ' + str(self.options["rtol"]),  verbose)
+        self.log_message('',                                                         verbose)
+        
+
+class _Radau5ODE(Radau_Common,Explicit_ODE):
     """
     Radau IIA fifth-order three-stages with step-size control and continuous output.
     Based on the FORTRAN code by E.Hairer and G.Wanner, which can be found here: 
@@ -555,9 +769,255 @@ class Radau5ODE(Radau_Common,Explicit_ODE):
         self.I3 = I3
         self.EIG = eig
 
-
-
 class Radau5DAE(Radau_Common,Implicit_ODE):
+    """
+    Radau IIA fifth-order three-stages with step-size control and 
+    continuous output. Based on the FORTRAN code RADAU5 by E.Hairer and 
+    G.Wanner, which can be found here: 
+    http://www.unige.ch/~hairer/software.html
+    
+    Details about the implementation (FORTRAN) can be found in the book,::
+    
+        Solving Ordinary Differential Equations II,
+        Stiff and Differential-Algebraic Problems
+        
+        Authors: E. Hairer and G. Wanner
+        Springer-Verlag, ISBN: 3-540-60452-9
+    
+    """
+    
+    def __init__(self, problem):
+        """
+        Initiates the solver.
+        
+            Parameters::
+            
+                problem     
+                            - The problem to be solved. Should be an instance
+                              of the 'Explicit_Problem' class.
+        """
+        Implicit_ODE.__init__(self, problem) #Calls the base class
+        
+        #Default values
+        self.options["inith"]    = 0.01
+        self.options["newt"]     = 7 #Maximum number of newton iterations
+        self.options["thet"]     = 1.e-3 #Boundary for re-calculation of jac
+        self.options["fnewt"]    = 0.0 #Stopping critera for Newtons Method
+        self.options["quot1"]    = 1.0 #Parameters for changing step-size (lower bound)
+        self.options["quot2"]    = 1.2 #Parameters for changing step-size (upper bound)
+        self.options["fac1"]     = 0.2 #Parameters for step-size selection (lower bound)
+        self.options["fac2"]     = 8.0 #Parameters for step-size selection (upper bound)
+        self.options["maxh"]     = N.inf #Maximum step-size.
+        self.options["safe"]     = 0.9 #Safety factor
+        self.options["atol"]     = 1.0e-6*N.ones(self.problem_info["dim"]) #Absolute tolerance
+        self.options["rtol"]     = 1.0e-6 #Relative tolerance
+        self.options["usejac"]   = True if self.problem_info["jac_fcn"] else False
+        self.options["maxsteps"] = 10000
+        
+        # - Statistic values
+        self.statistics["nsteps"]      = 0 #Number of steps
+        self.statistics["nfcn"]        = 0 #Number of function evaluations
+        self.statistics["njac"]        = 0 #Number of jacobian evaluations
+        self.statistics["njacfcn"]     = 0 #Number of function evaluations when evaluating the jacobian
+        self.statistics["errfail"]     = 0 #Number of step rejections
+        self.statistics["nlu"]         = 0 #Number of LU decompositions
+        self.statistics["nstepstotal"] = 0 #Number of total computed steps (may NOT be equal to nsteps+nerrfail)
+        self.statistics["nstateevents"]= 0 #Number of state events
+        self.statistics["ngevals"]     = 0 #Root evaluations
+        
+        #Solver support
+        self.supports["report_continuously"] = True
+        self.supports["interpolated_output"] = True
+        self.supports["state_events"] = True
+        
+        self._leny = len(self.y) #Dimension of the problem
+        self._type = '(implicit)'
+        self._event_info = None
+        
+    def initialize(self):
+        #Reset statistics
+        for k in self.statistics.keys():
+            self.statistics[k] = 0
+        
+    def set_problem_data(self):
+        if self.problem_info["state_events"]:
+            if self.problem_info["type"] == 1:
+                def event_func(t, y, yd):
+                    return self.problem.state_events(t, y, yd, self.sw)
+            else:
+                def event_func(t, y, yd):
+                    return self.problem.state_events(t, y, self.sw)
+            def f(t, y):
+                leny = self._leny
+                res = self.problem.res(t, y[:leny], y[leny:2*leny], self.sw)
+                return N.append(y[leny:2*leny],res)
+            self._f = f
+            self.event_func = event_func
+            self._event_info = [0] * self.problem_info["dimRoot"]
+            self.g_old = self.event_func(self.t, self.y, self.yd)
+        else:
+            def f(t, y):
+                leny = self._leny
+                res = self.problem.res(t, y[:leny], y[leny:2*leny])
+                return N.append(y[leny:2*leny],res)
+            self._f = f
+    
+    def interpolate(self, time, k=0):
+        y = N.empty(self._leny*2)
+        for i in range(self._leny*2):
+            y[i] = radau5.contr5(i+1, time, self.cont)
+        if k == 0:
+            return y[:self._leny]
+        elif k == 1:
+            return y[self._leny:2*self._leny]
+        
+    def _solout(self, nrsol, told, t, y, cont, lrc, irtrn):
+        """
+        This method is called after every successful step taken by Radau5
+        """
+        self.cont = cont #Saved to be used by the interpolation function.
+        
+        yd = y[self._leny:2*self._leny].copy()
+        y = y[:self._leny].copy()
+        if self.problem_info["state_events"]:
+            flag, t, y, yd = self.event_locator(told, t, y, yd)
+            #Convert to Fortram indicator.
+            if flag == ID_PY_EVENT: irtrn = -1
+        
+        if self._opts["report_continuously"]:
+            initialize_flag = self.report_solution(t, y, yd, self._opts)
+            if initialize_flag: irtrn = -1
+        else:
+            if self._opts["output_list"] == None:
+                self._tlist.append(t)
+                self._ylist.append(y)
+                self._ydlist.append(yd)
+            else:
+                output_list = self._opts["output_list"]
+                output_index = self._opts["output_index"]
+                try:
+                    while output_list[output_index] <= t:
+                        self._tlist.append(output_list[output_index])
+                        self._ylist.append(self.interpolate(output_list[output_index]))
+                        self._ydlist.append(self.interpolate(output_list[output_index], 1))
+    
+                        output_index += 1
+                except IndexError:
+                    pass
+                self._opts["output_index"] = output_index
+            
+        return irtrn
+        
+    def _mas_f(self, am):
+        #return N.array([[1]*self._leny+[0]*self._leny])
+        return self._mass_matrix
+        
+    def integrate(self, t, y, yd, tf, opts):
+        if self.usejac:
+            self.usejac=False
+            self.log_message("Jacobians are not currently supported, disabling.",NORMAL)
+        
+        ITOL  = 1 #Both atol and rtol are vectors
+        IJAC  = 1 if self.usejac else 0 #Switch for the jacobian, 0==NO JACOBIAN
+        MLJAC = self.problem_info["dim"]*2 #The jacobian is full
+        MUJAC = 0 #self.problem_info["dim"] #See MLJAC
+        IMAS  = 1 #The mass matrix is supplied
+        MLMAS = 0 #The mass matrix is only defined on the diagonal
+        MUMAS = 0 #The mass matrix is only defined on the diagonal
+        IOUT  = 1 #solout is called after every step
+        WORK  = N.array([0.0]*(5*((self.problem_info["dim"]*2)**2+12)+20)) #Work (double) vector
+        IWORK = N.array([0]*(3*(self.problem_info["dim"]*2)+20)) #Work (integer) vector
+        
+        #Setting work options
+        WORK[1] = self.safe
+        WORK[2] = self.thet
+        WORK[3] = self.fnewt
+        WORK[4] = self.quot1
+        WORK[5] = self.quot2
+        WORK[6] = self.maxh
+        WORK[7] = self.fac1
+        WORK[8] = self.fac2
+        
+        #Setting iwork options
+        IWORK[1] = self.maxsteps
+        IWORK[2] = self.newt
+        IWORK[4] = self._leny #Number of index 1 variables
+        IWORK[5] = self._leny #Number of index 2 variables
+        IWORK[8] = self._leny #M1
+        IWORK[9] = self._leny #M2
+        
+        #Dummy methods
+        mas_dummy = lambda t:x
+        jac_dummy = (lambda t:x) if not self.usejac else self.problem.jac
+        
+        #Check for initialization
+        if opts["initialize"]:
+            self.set_problem_data()  
+            self._tlist  = []
+            self._ylist  = []
+            self._ydlist = []
+        
+        #Store the opts
+        self._opts = opts
+        
+        #Create y = [y, yd]
+        y = N.append(y,yd)
+        #Create mass matrix
+        #self._mass_matrix = N.array([[1]*self._leny+[0]*self._leny])
+        self._mass_matrix = N.array([[0]*self._leny])
+        
+        atol = N.append(self.atol, self.atol)
+        
+        t, y, h, iwork, flag =  radau5.radau5(self._f, t, y.copy(), tf, self.inith, self.rtol*N.ones(self.problem_info["dim"]*2), atol, 
+                        ITOL, jac_dummy, IJAC, MLJAC, MUJAC, self._mas_f, IMAS, MLMAS, MUMAS, self._solout, IOUT, WORK, IWORK)
+        
+        #Checking return
+        if flag == 1:
+            flag = ID_PY_COMPLETE
+        elif flag == 2:
+            flag = ID_PY_EVENT
+        else:
+            raise Exception("Radau5 failed with flag %d"%flag)
+        
+        #Retrieving statistics
+        self.statistics["nsteps"]      += iwork[16]
+        self.statistics["nfcn"]        += iwork[13]
+        self.statistics["njac"]        += iwork[14]
+        self.statistics["nstepstotal"] += iwork[15]
+        self.statistics["errfail"]     += iwork[17]
+        self.statistics["nlu"]         += iwork[18]
+        
+        return flag, self._tlist, self._ylist, self._ydlist
+        
+    def state_event_info(self):
+        return self._event_info
+        
+    def set_event_info(self, event_info):
+        self._event_info = event_info
+    
+    def print_statistics(self, verbose=NORMAL):
+        """
+        Prints the run-time statistics for the problem.
+        """
+        self.log_message('Final Run Statistics: %s \n' % self.problem.name,        verbose)
+        
+        self.log_message(' Number of Steps                          : '+str(self.statistics["nsteps"]),          verbose)               
+        self.log_message(' Number of Function Evaluations           : '+str(self.statistics["nfcn"]),         verbose)
+        self.log_message(' Number of Jacobian Evaluations           : '+ str(self.statistics["njac"]),    verbose)
+        self.log_message(' Number of Error Test Failures            : '+ str(self.statistics["errfail"]),       verbose)
+        self.log_message(' Number of LU decompositions              : '+ str(self.statistics["nlu"]),       verbose)
+        if self.problem_info["state_events"]:
+            self.log_message(' Number of Root Evaluations               : '+ str(self.statistics["ngevals"]),        verbose)
+            self.log_message(' Number of State-Events                   : '+ str(self.statistics["nstateevents"]),   verbose)
+
+        
+        self.log_message('\nSolver options:\n',                                      verbose)
+        self.log_message(' Solver                  : Radau5 ' + self._type,          verbose)
+        self.log_message(' Tolerances (absolute)   : ' + str(self.options["atol"]),  verbose)
+        self.log_message(' Tolerances (relative)   : ' + str(self.options["rtol"]),  verbose)
+        self.log_message('',                                                         verbose)
+
+class _Radau5DAE(Radau_Common,Implicit_ODE):
     """
     Radau IIA fifth-order three-stages with step-size control and continuous output.
     Based on the FORTRAN code by E.Hairer and G.Wanner, which can be found here: 
